@@ -72,6 +72,10 @@ class TestSourcesListUpdate(unittest.TestCase):
             os.unlink(os.path.join(self.testdir, "sources.list"))
         apt_pkg.config.set("APT::Default-Release", "")
 
+    def tearDown(self):
+        if os.path.exists(os.path.join(self.testdir, "sources.list")):
+            os.unlink(os.path.join(self.testdir, "sources.list"))
+
     def test_sources_list_with_nothing(self):
         """
         test sources.list rewrite with nothing in it
@@ -92,8 +96,9 @@ deb http://archive.ubuntu.com/ubuntu gutsy-updates main restricted
 deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
 """)
 
+    @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
     @mock.patch("DistUpgrade.DistUpgradeController.get_distro")
-    def test_sources_list_rewrite(self, mock_get_distro):
+    def test_sources_list_rewrite(self, mock_get_distro, mock_sourcesListEntryDownloadable):
         """
         test regular sources.list rewrite
         """
@@ -107,8 +112,10 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
                                                           "Ubuntu Feisty Fawn",
                                                           "7.04")
         d.openCache(lock=False)
+        mock_sourcesListEntryDownloadable.return_value = True
         res = d.updateSourcesList()
         self.assertTrue(mock_get_distro.called)
+        self.assertTrue(mock_sourcesListEntryDownloadable.called)
         self.assertTrue(res)
 
         # now test the result
@@ -127,7 +134,40 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security universe
              apt_pkg.config.find_file("Dir::Etc::sourcelist") + ".in",
              apt_pkg.config.find_file("Dir::Etc::sourcelist") + ".distUpgrade"
              ]))
-             
+
+    @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
+    @mock.patch("DistUpgrade.DistUpgradeController.get_distro")
+    def test_sources_list_rewrite_no_network(self, mock_get_distro, mock_sourcesListEntryDownloadable):
+        """
+        test sources.list rewrite no network
+        """
+        shutil.copy(os.path.join(self.testdir, "sources.list.in"),
+                    os.path.join(self.testdir, "sources.list"))
+        apt_pkg.config.set("Dir::Etc::sourcelist", "sources.list")
+        v = DistUpgradeViewNonInteractive()
+        d = DistUpgradeController(v, datadir=self.testdir)
+        d.config.set("Distro", "BaseMetaPkgs", "ubuntu-minimal")
+        mock_get_distro.return_value = UbuntuDistribution("Ubuntu", "feisty",
+                                                          "Ubuntu Feisty Fawn",
+                                                          "7.04")
+        d.openCache(lock=False)
+        mock_sourcesListEntryDownloadable.return_value = False
+        res = d.updateSourcesList()
+        self.assertTrue(mock_get_distro.called)
+        self.assertTrue(mock_sourcesListEntryDownloadable.called)
+        self.assertTrue(res)
+
+        # now test the result
+        #print(open(os.path.join(self.testdir,"sources.list")).read())
+        self._verifySources("""
+# main repo
+# deb http://archive.ubuntu.com/ubuntu/ feisty main restricted multiverse universe
+# deb http://de.archive.ubuntu.com/ubuntu/ feisty main restricted multiverse
+# deb-src http://archive.ubuntu.com/ubuntu/ feisty main restricted multiverse
+# deb http://security.ubuntu.com/ubuntu/ feisty-security main restricted
+# deb http://security.ubuntu.com/ubuntu/ feisty-security universe
+""")
+
     @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController.abort")
     @mock.patch("DistUpgrade.DistUpgradeController.get_distro")
     def test_double_check_source_distribution_reject(self, mock_abort, mock_get_distro):
@@ -145,10 +185,10 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security universe
         mock_get_distro.return_value = UbuntuDistribution("Ubuntu", "feisty",
                                                           "Ubuntu Feisty Fawn",
                                                           "7.04")
-        
+
         class AbortException(Exception):
             """Exception"""
-                
+
         mock_abort.side_effect = AbortException
         d.openCache(lock=False)
         with self.assertRaises(AbortException):
@@ -157,8 +197,9 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security universe
         self.assertTrue(mock_get_distro.called)
         self.assertTrue(v.askYesNoQuestion.called)
 
+    @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
     @mock.patch("DistUpgrade.DistUpgradeController.get_distro")
-    def test_double_check_source_distribution_continue(self, mock_get_distro):
+    def test_double_check_source_distribution_continue(self, mock_get_distro, mock_sourcesListEntryDownloadable):
         """
         test that an upgrade from feisty with a sources.list containing
         hardy asks a question, and if continued, does something.
@@ -174,13 +215,15 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security universe
                                                           "Ubuntu Feisty Fawn",
                                                           "7.04")
         d.openCache(lock=False)
+        mock_sourcesListEntryDownloadable.return_value = True
         res = d.updateSourcesList()
         self.assertTrue(mock_get_distro.called)
+        self.assertTrue(mock_sourcesListEntryDownloadable.called)
         self.assertTrue(res)
 
         # now test the result
         #print(open(os.path.join(self.testdir,"sources.list")).read())
-        
+
         # The result here is not really all that helpful, hence we
         # added the question in the first place. But still better to
         # check what it does than to not check it.
@@ -195,6 +238,7 @@ deb-src http://uk.archive.ubuntu.com/ubuntu/ hardy main restricted multiverse
 # deb http://security.ubuntu.com/ubuntu/ hardy-security universe
 
 deb http://archive.ubuntu.com/ubuntu/ gutsy main
+
 """)
         # check that the backup file was created correctly
         self.assertEqual(0, subprocess.call(
@@ -203,8 +247,9 @@ deb http://archive.ubuntu.com/ubuntu/ gutsy main
              apt_pkg.config.find_file("Dir::Etc::sourcelist") + ".distUpgrade"
              ]))
 
+    @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
     @mock.patch("DistUpgrade.DistUpgradeController.get_distro")
-    def test_sources_list_inactive_mirror(self, mock_get_distro):
+    def test_sources_list_inactive_mirror(self, mock_get_distro, mock_sourcesListEntryDownloadable):
         """
         test sources.list rewrite of an obsolete mirror
         """
@@ -218,8 +263,10 @@ deb http://archive.ubuntu.com/ubuntu/ gutsy main
                                                           "Ubuntu Feisty Fawn",
                                                           "7.04")
         d.openCache(lock=False)
+        mock_sourcesListEntryDownloadable.return_value = True
         res = d.updateSourcesList()
         self.assertTrue(mock_get_distro.called)
+        self.assertTrue(mock_sourcesListEntryDownloadable.called)
         self.assertTrue(res)
 
         # now test the result
@@ -311,8 +358,9 @@ deb http://security.ubuntu.com/ubuntu gutsy-security main restricted # auto gene
 deb http://archive.canonical.com/ubuntu gutsy partner
 """)
 
+    @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
     @mock.patch("DistUpgrade.DistUpgradeController.get_distro")
-    def test_extras_removal(self, mock_get_distro):
+    def test_extras_removal(self, mock_get_distro, mock_sourcesListEntryDownloadable):
         """
         test removal of extras.ubuntu.com archives
         """
@@ -328,7 +376,9 @@ deb http://archive.canonical.com/ubuntu gutsy partner
                                                           "Ubuntu Feisty Fawn",
                                                           "7.04")
         d.openCache(lock=False)
+        mock_sourcesListEntryDownloadable.return_value = True
         res = d.updateSourcesList()
+        self.assertTrue(mock_sourcesListEntryDownloadable.called)
         self.assertTrue(res)
 
         sources_file = apt_pkg.config.find_file("Dir::Etc::sourcelist")
@@ -338,7 +388,8 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
 
 """)
 
-    def test_powerpc_transition(self):
+    @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
+    def test_powerpc_transition(self, mock_sourcesListEntryDownloadable):
         """
         test transition of powerpc to ports.ubuntu.com
         """
@@ -351,7 +402,9 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
         v = DistUpgradeViewNonInteractive()
         d = DistUpgradeController(v, datadir=self.testdir)
         d.openCache(lock=False)
+        mock_sourcesListEntryDownloadable.return_value = True
         res = d.updateSourcesList()
+        self.assertTrue(mock_sourcesListEntryDownloadable.called)
         self.assertTrue(res)
         # now test the result
         self._verifySources("""
@@ -362,7 +415,8 @@ deb http://ports.ubuntu.com/ubuntu-ports/ gutsy-security main restricted univers
 """)
         apt_pkg.config.set("APT::Architecture", arch)
 
-    def test_sparc_transition(self):
+    @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
+    def test_sparc_transition(self, mock_sourcesListEntryDownloadable):
         """
         test transition of sparc to ports.ubuntu.com
         """
@@ -377,7 +431,9 @@ deb http://ports.ubuntu.com/ubuntu-ports/ gutsy-security main restricted univers
         d.fromDist = "gutsy"
         d.toDist = "hardy"
         d.openCache(lock=False)
+        mock_sourcesListEntryDownloadable.return_value = True
         res = d.updateSourcesList()
+        self.assertTrue(mock_sourcesListEntryDownloadable.called)
         self.assertTrue(res)
         # now test the result
         self._verifySources("""
@@ -393,7 +449,7 @@ deb http://ports.ubuntu.com/ubuntu-ports/ hardy-security main restricted univers
         v = DistUpgradeViewNonInteractive()
         d = DistUpgradeController(v, datadir=self.testdir)
         for scheme in ["http"]:
-            entry = "deb %s://archive.ubuntu.com/ubuntu/ precise main universe restricted multiverse" % scheme
+            entry = "deb %s://archive.ubuntu.com/ubuntu/ focal main universe restricted multiverse" % scheme
             self.assertTrue(d._sourcesListEntryDownloadable(SourceEntry(entry)),
                             "entry '%s' not downloadable" % entry)
             entry = "deb %s://archive.ubuntu.com/ubuntu/ warty main universe restricted multiverse" % scheme
@@ -405,8 +461,6 @@ deb http://ports.ubuntu.com/ubuntu-ports/ hardy-security main restricted univers
 
     def testEOL2EOLUpgrades(self):
         " test upgrade from EOL release to EOL release "
-        v = DistUpgradeViewNonInteractive()
-        d = DistUpgradeController(v, datadir=self.testdir)
         shutil.copy(os.path.join(self.testdir, "sources.list.EOL"),
                     os.path.join(self.testdir, "sources.list"))
         apt_pkg.config.set("Dir::Etc::sourceparts",
@@ -435,25 +489,50 @@ deb http://old-releases.ubuntu.com/ubuntu hoary-security main restricted univers
         # data center, unlike most mirrors.  This lets this test pass when
         # when run in their Jenkins test environment.
         os.environ["LANG"] = "en_US.UTF-8"
-        v = DistUpgradeViewNonInteractive()
-        d = DistUpgradeController(v, datadir=self.testdir)
         shutil.copy(os.path.join(self.testdir, "sources.list.EOL2Supported"),
                     os.path.join(self.testdir, "sources.list"))
         apt_pkg.config.set("Dir::Etc::sourceparts",
                            os.path.join(self.testdir, "sources.list.d"))
         v = DistUpgradeViewNonInteractive()
         d = DistUpgradeController(v, datadir=self.testdir)
-        d.fromDist = "oneiric"
-        d.toDist = "precise"
+        d.fromDist = "eoan"
+        d.toDist = "focal"
         d.openCache(lock=False)
         res = d.updateSourcesList()
         self.assertTrue(res)
         self._verifySources("""
 # main repo
-deb http://us.archive.ubuntu.com/ubuntu precise main restricted multiverse universe
-deb-src http://us.archive.ubuntu.com/ubuntu precise main restricted multiverse
+deb http://us.archive.ubuntu.com/ubuntu focal main restricted multiverse universe
+deb-src http://us.archive.ubuntu.com/ubuntu focal main restricted multiverse
 
-deb http://us.archive.ubuntu.com/ubuntu precise-security main restricted universe multiverse
+deb http://us.archive.ubuntu.com/ubuntu focal-security main restricted universe multiverse
+""")
+
+    @unittest.skipUnless(url_downloadable(
+        "https://mirrors.kernel.org/ubuntu", logging.debug),
+        "Could not reach mirror")
+    def testSupportedWithHttpsMirrorUpgrade(self):
+        " test upgrade from to a supported release with https mirror "
+        # Use mirrors.kernel.org because it supports https, when the main
+        # archive does we should switch that.
+        os.environ["LANG"] = "en_US.UTF-8"
+        shutil.copy(os.path.join(self.testdir, "sources.list.https"),
+                    os.path.join(self.testdir, "sources.list"))
+        apt_pkg.config.set("Dir::Etc::sourceparts",
+                           os.path.join(self.testdir, "sources.list.d"))
+        v = DistUpgradeViewNonInteractive()
+        d = DistUpgradeController(v, datadir=self.testdir)
+        d.fromDist = "xenial"
+        d.toDist = "bionic"
+        d.openCache(lock=False)
+        res = d.updateSourcesList()
+        self.assertTrue(res)
+        self._verifySources("""
+# main repo
+deb https://mirrors.kernel.org/ubuntu bionic main restricted multiverse universe
+deb-src https://mirrors.kernel.org/ubuntu bionic main restricted multiverse
+
+deb https://mirrors.kernel.org/ubuntu bionic-security main restricted universe multiverse
 """)
 
     def testEOL2SupportedUpgrade(self):
@@ -467,20 +546,21 @@ deb http://us.archive.ubuntu.com/ubuntu precise-security main restricted univers
                            os.path.join(self.testdir, "sources.list.d"))
         v = DistUpgradeViewNonInteractive()
         d = DistUpgradeController(v, datadir=self.testdir)
-        d.fromDist = "oneiric"
-        d.toDist = "precise"
+        d.fromDist = "eoan"
+        d.toDist = "focal"
         d.openCache(lock=False)
         res = d.updateSourcesList()
         self.assertTrue(res)
         self._verifySources("""
 # main repo
-deb http://archive.ubuntu.com/ubuntu precise main restricted multiverse universe
-deb-src http://archive.ubuntu.com/ubuntu precise main restricted multiverse
+deb http://archive.ubuntu.com/ubuntu focal main restricted multiverse universe
+deb-src http://archive.ubuntu.com/ubuntu focal main restricted multiverse
 
-deb http://archive.ubuntu.com/ubuntu precise-security main restricted universe multiverse
+deb http://archive.ubuntu.com/ubuntu focal-security main restricted universe multiverse
 """)
 
-    def test_partner_update(self):
+    @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
+    def test_partner_update(self, mock_sourcesListEntryDownloadable):
         """
         test transition partner repository updates
         """
@@ -491,7 +571,9 @@ deb http://archive.ubuntu.com/ubuntu precise-security main restricted universe m
         v = DistUpgradeViewNonInteractive()
         d = DistUpgradeController(v, datadir=self.testdir)
         d.openCache(lock=False)
+        mock_sourcesListEntryDownloadable.return_value = True
         res = d.updateSourcesList()
+        self.assertTrue(mock_sourcesListEntryDownloadable.called)
         self.assertTrue(res)
 
         # now test the result
@@ -504,9 +586,10 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted universe m
 deb http://archive.canonical.com/ubuntu gutsy partner
 """)
 
-    def test_private_ppa_transition(self):
-        if "RELEASE_UPRADER_ALLOW_THIRD_PARTY" in os.environ:
-            del os.environ["RELEASE_UPRADER_ALLOW_THIRD_PARTY"]
+    @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
+    def test_private_ppa_transition(self, mock_sourcesListEntryDownloadable):
+        if "RELEASE_UPGRADER_ALLOW_THIRD_PARTY" in os.environ:
+            del os.environ["RELEASE_UPGRADER_ALLOW_THIRD_PARTY"]
         shutil.copy(
             os.path.join(self.testdir,
                          "sources.list.commercial-ppa-uploaders"),
@@ -516,7 +599,9 @@ deb http://archive.canonical.com/ubuntu gutsy partner
         v = DistUpgradeViewNonInteractive()
         d = DistUpgradeController(v, datadir=self.testdir)
         d.openCache(lock=False)
+        mock_sourcesListEntryDownloadable.return_value = True
         res = d.updateSourcesList()
+        self.assertTrue(mock_sourcesListEntryDownloadable.called)
         self.assertTrue(res)
 
         # now test the result
@@ -533,7 +618,8 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted universe m
 deb https://user:pass@private-ppa.launchpad.net/commercial-ppa-uploaders gutsy main
 """)
 
-    def test_apt_cacher_and_apt_bittorent(self):
+    @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
+    def test_apt_cacher_and_apt_bittorent(self, mock_sourcesListEntryDownloadable):
         """
         test transition of apt-cacher/apt-torrent uris
         """
@@ -544,7 +630,9 @@ deb https://user:pass@private-ppa.launchpad.net/commercial-ppa-uploaders gutsy m
         v = DistUpgradeViewNonInteractive()
         d = DistUpgradeController(v, datadir=self.testdir)
         d.openCache(lock=False)
+        mock_sourcesListEntryDownloadable.return_value = True
         res = d.updateSourcesList()
+        self.assertTrue(mock_sourcesListEntryDownloadable.called)
         self.assertTrue(res)
 
         # now test the result
@@ -593,9 +681,10 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
 # deb http://ppa.launchpad.net/random-ppa quantal main # ppa of Víctor R. Ruiz (vrruiz) disabled on upgrade to gutsy
 """)
 
-    def test_local_mirror(self):
+    @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
+    def test_local_mirror(self, mock_sourcesListEntryDownloadable):
         """
-        test that a local mirror with official -backports works (LP:# 1067393)
+        test that a local mirror with official -backports works (LP: #1067393)
         """
         shutil.copy(os.path.join(self.testdir, "sources.list.local"),
                     os.path.join(self.testdir, "sources.list"))
@@ -603,7 +692,9 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
         v = DistUpgradeViewNonInteractive()
         d = DistUpgradeController(v, datadir=self.testdir)
         d.openCache(lock=False)
+        mock_sourcesListEntryDownloadable.return_value = True
         res = d.updateSourcesList()
+        self.assertTrue(mock_sourcesListEntryDownloadable.called)
         self.assertTrue(res)
 
         # verify it
@@ -614,7 +705,8 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
 deb http://archive.ubuntu.com/ubuntu gutsy-backports main restricted universe multiverse
 """)
 
-    def test_disable_proposed(self):
+    @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
+    def test_disable_proposed(self, mock_sourcesListEntryDownloadable):
         """
         Test that proposed is disabled when upgrading to a development
         release.
@@ -628,7 +720,9 @@ deb http://archive.ubuntu.com/ubuntu gutsy-backports main restricted universe mu
         options.devel_release = True
         d = DistUpgradeController(v, options, datadir=self.testdir)
         d.openCache(lock=False)
+        mock_sourcesListEntryDownloadable.return_value = True
         res = d.updateSourcesList()
+        self.assertTrue(mock_sourcesListEntryDownloadable.called)
         self.assertTrue(res)
 
         self._verifySources("""
